@@ -35,7 +35,6 @@ _STRING_FIELDS = (
     models.CharField,
     models.TextField,
     models.GenericIPAddressField,
-    models.UUIDField,
 )
 _NUMBER_FIELDS = (
     models.DecimalField,
@@ -49,6 +48,7 @@ _FIELD_TYPE_MAP = {
     models.NullBooleanField: BooleanType,
     models.DateTimeField: DateTimeType,
     models.DateField: DateType,
+    models.UUIDField: UnknownType,
     **{f: StringType for f in _STRING_FIELDS},
     **{f: NumberType for f in _NUMBER_FIELDS},
 }
