@@ -3,7 +3,9 @@ import re
 from functools import lru_cache
 
 import dateutil.parser
+from dateutil import relativedelta
 from django.conf import settings
+from django.core.serializers.json import DjangoJSONEncoder
 from django.utils import dateparse, html, timezone
 
 from .common import all_subclasses, get_optimal_decimal_places
@@ -64,17 +66,20 @@ class BaseType(metaclass=TypeMeta):
         return value
 
     @classmethod
-    def parse(cls, lookup, value, choices):
+    def parse(cls, value, choices):
+        try:
+            return cls._parse(value, choices), None
+        except Exception as e:
+            return None, str(e) if str(e) else repr(e)
+
+    @classmethod
+    def parse_lookup(cls, lookup, value, choices):
         lookups = cls.lookups
         if lookup not in lookups:
             return None, f"Bad lookup '{lookup}' expected {lookups}"
         else:
             type_ = TYPES[lookups[lookup]]
-            try:
-                return type_._parse(value, choices), None
-            except Exception as e:
-                err_message = str(e) if str(e) else repr(e)
-                return None, err_message
+            return type_.parse(value, choices)
 
     @staticmethod
     def get_format_hints(name, data):
@@ -188,6 +193,31 @@ class DurationType(BaseType):
 
 
 class DateTypeMixin:
+    _clause_str = r"(\w{3,})([+-=])(\d+) *"
+    _clause = re.compile(_clause_str)
+    _clauses = re.compile(fr"^({_clause_str})+$")
+
+    _clause_types = {
+        "yea": "year",
+        "mont": "month",
+        "day": "day",
+        "hour": "hour",
+        "min": "minute",
+        "sec": "second",
+        "mic": "microsecond",
+        "mond": relativedelta.MO,
+        "tue": relativedelta.TU,
+        "wed": relativedelta.WE,
+        "thu": relativedelta.TH,
+        "fri": relativedelta.FR,
+        "sat": relativedelta.SA,
+        "sun": relativedelta.SU,
+    }
+
+    for c1 in _clause_types:
+        for c2 in _clause_types:
+            assert c1 == c2 or not c1.startswith(c2), c2
+
     @classmethod
     def _lookups(cls):
         return {
@@ -202,20 +232,61 @@ class DateTypeMixin:
 
     @classmethod
     def _parse(cls, value, choices):
+        value = value.strip()
+
         d8 = r"(\d{8})"
         d422 = r"(\d{4}[^\d]*\d{2}[^\d]*\d{2})"
         if re.match(r"[^\d]*(" + d8 + "|" + d422 + ")", value):
             # looks like some kinda iso date, roll with the defaults
             res = dateutil.parser.parse(value)
         else:
-            res = {
-                dateutil.parser.parse(value, dayfirst=False, yearfirst=False),
-                dateutil.parser.parse(value, dayfirst=True, yearfirst=False),
-                dateutil.parser.parse(value, dayfirst=False, yearfirst=True),
-                dateutil.parser.parse(value, dayfirst=True, yearfirst=True),
-            }
-            assert len(res) == 1, "Ambiguous value"
-            res = res.pop()
+            try:
+                # not an iso date, let the parser do it thing, but check for ambiguities
+                res = {
+                    dateutil.parser.parse(value, dayfirst=False, yearfirst=False),
+                    dateutil.parser.parse(value, dayfirst=True, yearfirst=False),
+                    dateutil.parser.parse(value, dayfirst=False, yearfirst=True),
+                    dateutil.parser.parse(value, dayfirst=True, yearfirst=True),
+                }
+                assert len(res) == 1, "Ambiguous value"
+                res = res.pop()
+            except dateutil.parser.ParserError:
+                # failing that must be relative delta stuff
+                res = timezone.now()
+                assert cls._clauses.match(value), "Unrecognized value"
+
+                for match in cls._clause.finditer(value):
+                    field, op, val = match.groups()
+                    val = int(val)
+
+                    for prefix, arg in cls._clause_types.items():
+                        if field.startswith(prefix):
+                            break
+                    else:
+                        assert False, f"Unrecognized field {field}"
+
+                    if isinstance(arg, str):
+                        if op == "+":
+                            kwargs = {f"{arg}s": val}
+                        elif op == "-":
+                            kwargs = {f"{arg}s": -val}
+                        else:
+                            if arg in ["year", "month", "day"]:
+                                assert val > 0, f"Can't set {field} to {val}"
+                            kwargs = {arg: val}
+                    else:
+                        assert val > 0, f"Can't set {field} to {val}"
+                        if op == "+":
+                            kwargs = {"weekday": arg(val)}
+                        elif op == "-":
+                            kwargs = {"weekday": arg(-val)}
+                        else:
+                            # =
+                            assert False, f"{op} not supported for {field}"
+
+                    res += relativedelta.relativedelta(**kwargs)
+                return res
+
         return timezone.make_aware(res)
 
 
@@ -226,17 +297,24 @@ class DateTimeType(DateTypeMixin, BaseType):
     def _parse(cls, value, choices):
         if value.lower().strip() == "now":
             return timezone.now()
-        return super()._parse(value, choices)
+        else:
+            return super()._parse(value, choices)
 
     @staticmethod
     def _get_formatter(choices):
         assert not choices
         if settings.USE_TZ:
             return (
-                lambda value: None if value is None else str(timezone.make_naive(value))
+                lambda value: None
+                if value is None
+                else str(timezone.make_naive(value).replace(microsecond=0))
             )
         else:
-            return lambda value: None if value is None else str(value)
+            return (
+                lambda value: None
+                if value is None
+                else str(value.replace(microsecond=0))
+            )
 
 
 class DateType(DateTypeMixin, BaseType):
@@ -245,8 +323,10 @@ class DateType(DateTypeMixin, BaseType):
     @classmethod
     def _parse(cls, value, choices):
         if value.lower().strip() == "today":
-            return timezone.now().date()
-        return super()._parse(value, choices).date()
+            res = timezone.now()
+        else:
+            res = super()._parse(value, choices)
+        return res.date()
 
     @staticmethod
     def _get_formatter(choices):
@@ -394,7 +474,9 @@ class ArrayTypeMixin:
         return (
             lambda value: None
             if value is None
-            else json.dumps([element_formatter(v) for v in value])
+            else json.dumps(
+                [element_formatter(v) for v in value], cls=DjangoJSONEncoder
+            )
         )
 
     @classmethod
@@ -414,7 +496,7 @@ class ArrayTypeMixin:
         value = _json_loads(value)
         if not isinstance(value, list):
             raise ValueError("Expected a list")
-        return [cls.element_type._parse(v, choices) for v in value]
+        return [cls.element_type._parse(v, choices)[0] for v in value]
 
 
 class StringArrayType(ArrayTypeMixin, BaseType):
