@@ -38,6 +38,10 @@ def _get_query_data(bound_query):
             for filter_ in bound_query.filters
         ],
         "filterErrors": [filter_.err_message for filter_ in bound_query.filters],
+        "parsedFilterValues": [
+            filter_.orm_bound_field.get_formatter()(filter_.parsed)
+            for filter_ in bound_query.filters
+        ],
         "fields": [
             {
                 "pathStr": field.path_str,
