@@ -1,6 +1,7 @@
-import React from "react";
+import React, { useContext } from "react";
 import "./App.css";
 import { TLink, SLink, Overlay } from "./Util";
+import { ShowContextMenu } from "./ContextMenu";
 
 function Spacer(props) {
   const { spaces } = props;
@@ -51,15 +52,24 @@ function HeadCell(props) {
   );
 }
 
-const DataCell = React.memo((props) => {
-  const { modelField, className, span, value, formatHint } = props;
+function DataCell(props) {
+  const {
+    modelField,
+    className,
+    span,
+    value,
+    formatHint,
+    query,
+    pathStr,
+    fullRow,
+  } = props;
   let formattedValue;
   if (value === undefined) {
     formattedValue = "";
   } else if (value === null) {
     formattedValue = null;
-  } else if (modelField.type === "html" && value) {
-    formattedValue = <div dangerouslySetInnerHTML={{ __html: value }} />;
+  } else if (modelField.type === "html") {
+    formattedValue = value;
   } else if (modelField.type === "number") {
     if (
       value > formatHint.highCutOff ||
@@ -73,12 +83,40 @@ const DataCell = React.memo((props) => {
   } else {
     formattedValue = String(value);
   }
+
+  const showContextMenu = useContext(ShowContextMenu);
+
+  function onContextMenu(e) {
+    showContextMenu(e, [
+      navigator.clipboard && {
+        name: "Copy",
+        fn: () => navigator.clipboard.writeText(formattedValue),
+      },
+      modelField.concrete && {
+        name: "Filter",
+        fn: () => query.addExactFilter(pathStr, value),
+      },
+      fullRow && {
+        name: "Drill down",
+        fn: () => query.drillDown(fullRow),
+      },
+    ]);
+  }
+
   return (
-    <td className={modelField.type + " " + className || ""} colSpan={span || 1}>
-      {formattedValue}
+    <td
+      className={modelField.type + " " + className || ""}
+      colSpan={span || 1}
+      onContextMenu={onContextMenu}
+    >
+      {modelField.type === "html" ? (
+        <div dangerouslySetInnerHTML={{ __html: value }} />
+      ) : (
+        formattedValue
+      )}
     </td>
   );
-});
+}
 
 function VTableHeadRow(props) {
   const { fields, query, classNameFirst, className } = props;
@@ -92,16 +130,27 @@ function VTableHeadRow(props) {
 }
 
 function VTableBodyRow(props) {
-  const { fields, query, classNameFirst, className, row, formatHints } = props;
+  const {
+    fields,
+    query,
+    classNameFirst,
+    className,
+    row,
+    formatHints,
+    fullRow,
+  } = props;
   return fields.map((field, i) => {
     if (row)
       return (
         <DataCell
+          {...{ query }}
+          pathStr={field.pathStr}
           key={field.pathStr}
           value={row[field.pathStr]}
           className={`${i ? "" : classNameFirst} ${className}`}
           modelField={query.getField(field.pathStr)}
           formatHint={formatHints[field.pathStr]}
+          fullRow={fullRow}
         />
       );
     else
@@ -121,10 +170,12 @@ function HTableRow(props) {
       <HeadCell {...{ query, field }} />
       {data.map((col, key) => (
         <DataCell
-          {...{ key, span, className }}
+          {...{ key, span, className, query }}
           value={col[field.pathStr]}
           modelField={query.getField(field.pathStr)}
           formatHint={formatHints[field.pathStr]}
+          fullRow={col}
+          pathStr={field.pathStr}
         />
       ))}
     </>
@@ -181,12 +232,14 @@ function Results(props) {
                 <VTableBodyRow
                   {...{ query, row, formatHints }}
                   fields={query.rowFields()}
+                  fullRow={row}
                 />
                 {body.map((table, key) => (
                   <VTableBodyRow
                     {...{ key, query, formatHints }}
                     fields={query.resFields()}
                     row={table[rowIndex]}
+                    fullRow={{ ...row, ...cols[key] }}
                     classNameFirst="LeftBorder"
                   />
                 ))}
