@@ -1,6 +1,7 @@
 from datetime import date, datetime, timedelta
 
 import pytest
+import time_machine
 from django.http import QueryDict
 from django.utils import timezone
 
@@ -18,14 +19,13 @@ from data_browser.types import (
     IsNullType,
     NumberChoiceType,
     NumberType,
+    RegexType,
     StringArrayType,
     StringChoiceType,
     StringType,
     UnknownType,
 )
 
-from .util import ANY
-
 
 @pytest.fixture
 def query():
@@ -131,6 +131,15 @@ def bound_query(query, orm_models):
     return BoundQuery.bind(query, orm_models)
 
 
+def parse_helper(type_, choices, value, expected):
+    res, err = type_.parse(value, choices)
+    if res is None:
+        assert isinstance(err, str)
+    else:
+        assert res == expected
+        assert err is None
+
+
 class TestQuery:
     def test_from_request(self, query):
         q = Query.from_request(
@@ -305,17 +314,23 @@ class TestBoundQuery:
 
 class TestType:
     def test_repr(self):
-        assert repr(StringType) == f"StringType"
+        assert repr(StringType) == "StringType"
+
+
+class TestRegexType:
+    @pytest.mark.django_db
+    @pytest.mark.parametrize(
+        "value,expected",
+        [
+            (".*", ".*"),
+            ("\\", None),
+        ],
+    )
+    def test_parse(self, value, expected):
+        parse_helper(RegexType, None, value, expected)
 
 
 class TestStringType:
-    @pytest.mark.django_db
-    def test_validate(self):
-        assert StringType.parse("contains", "hello", None) == ("hello", None)
-        assert StringType.parse("pontains", "hello", None) == (None, ANY(str))
-        assert StringType.parse("regex", ".*", None) == (".*", None)
-        assert StringType.parse("regex", "\\", None) == (None, ANY(str))
-
     def test_default_lookup(self):
         assert StringType.default_lookup == "equals"
 
@@ -325,12 +340,15 @@ class TestStringType:
 
 
 class TestNumberType:
-    def test_validate(self):
-        assert NumberType.parse("gt", "6.1", None) == (6.1, None)
-        assert NumberType.parse("pontains", "6.1", None) == (None, ANY(str))
-        assert NumberType.parse("gt", "hello", None) == (None, ANY(str))
-        assert NumberType.parse("is_null", "IsNull", None) == (True, None)
-        assert NumberType.parse("is_null", "hello", None) == (None, ANY(str))
+    @pytest.mark.parametrize(
+        "value,expected",
+        [
+            ("6.1", 6.1),
+            ("hello", None),
+        ],
+    )
+    def test_parse(self, value, expected):
+        parse_helper(NumberType, None, value, expected)
 
     def test_default_lookup(self):
         assert NumberType.default_lookup == "equals"
@@ -340,50 +358,59 @@ class TestNumberType:
         assert NumberType.get_formatter(None)(None) is None
 
 
+def dt(Y, M, D, h=0, m=0, s=0):
+    return datetime(Y, M, D, h, m, s, tzinfo=timezone.utc)
+
+
 class TestDateTimeType:
-    def test_validate(self):
-        assert DateTimeType.parse("gt", "2018-03-20T22:31:23", None) == (
-            ANY(datetime),
-            None,
-        )
-        assert DateTimeType.parse("gt", "hello", None) == (None, ANY(str))
-        assert DateTimeType.parse("pontains", "2018-03-20T22:31:23", None) == (
-            None,
-            ANY(str),
-        )
-        assert DateTimeType.parse("is_null", "IsNull", None) == (True, None)
-        assert DateTimeType.parse("is_null", "hello", None) == (None, ANY(str))
-        assert DateTimeType.parse("gt", "now", None) == (ANY(datetime), None)
-
-        assert DateTimeType.parse("gt", "11-11-2018", None) == (ANY(datetime), None)
-        assert DateTimeType.parse("gt", "11-22-2018", None) == (ANY(datetime), None)
-        assert DateTimeType.parse("gt", "21-12-2018", None) == (ANY(datetime), None)
-        assert DateTimeType.parse("gt", "11-12-2018", None) == (None, ANY(str))
-        assert DateTimeType.parse("gt", "21-22-2018", None) == (None, ANY(str))
-
-        assert DateTimeType.parse("gt", "2018-11-11", None) == (ANY(datetime), None)
-        assert DateTimeType.parse("gt", "2018-11-22", None) == (ANY(datetime), None)
-        assert DateTimeType.parse("gt", "2018-21-12", None) == (None, ANY(str))
-        assert DateTimeType.parse("gt", "2018-11-12", None) == (ANY(datetime), None)
-        assert DateTimeType.parse("gt", "2018-21-22", None) == (None, ANY(str))
-
-        assert DateTimeType.parse("gt", "20181111", None) == (ANY(datetime), None)
-        assert DateTimeType.parse("gt", "20181122", None) == (ANY(datetime), None)
-        assert DateTimeType.parse("gt", "20182112", None) == (None, ANY(str))
-        assert DateTimeType.parse("gt", "20181112", None) == (ANY(datetime), None)
-        assert DateTimeType.parse("gt", "20182122", None) == (None, ANY(str))
-
-        assert DateTimeType.parse("gt", "181111", None) == (None, ANY(str))
-        assert DateTimeType.parse("gt", "181122", None) == (None, ANY(str))
-        assert DateTimeType.parse("gt", "182112", None) == (None, ANY(str))
-        assert DateTimeType.parse("gt", "181112", None) == (None, ANY(str))
-        assert DateTimeType.parse("gt", "182122", None) == (None, ANY(str))
-
-        assert DateTimeType.parse("gt", "111118", None) == (None, ANY(str))
-        assert DateTimeType.parse("gt", "112218", None) == (ANY(datetime), None)
-        assert DateTimeType.parse("gt", "211218", None) == (None, ANY(str))
-        assert DateTimeType.parse("gt", "111218", None) == (None, ANY(str))
-        assert DateTimeType.parse("gt", "212218", None) == (None, ANY(str))
+    @time_machine.travel("2020-12-13 09:42:53", tick=False)
+    @pytest.mark.parametrize(
+        "value,expected",
+        [
+            ("2018-03-20T22:31:23", dt(2018, 3, 20, 22, 31, 23)),
+            ("hello", None),
+            ("now", dt(2020, 12, 13, 9, 42, 53)),
+            # dateutil.parser
+            ("11-11-2018", dt(2018, 11, 11)),
+            ("11-22-2018", dt(2018, 11, 22)),
+            ("21-12-2018", dt(2018, 12, 21)),
+            ("11-12-2018", None),
+            ("21-22-2018", None),
+            ("2018-11-11", dt(2018, 11, 11)),
+            ("2018-11-22", dt(2018, 11, 22)),
+            ("2018-21-12", None),
+            ("2018-11-12", dt(2018, 11, 12)),
+            ("2018-21-22", None),
+            ("20181111", dt(2018, 11, 11)),
+            ("20181122", dt(2018, 11, 22)),
+            ("20182112", None),
+            ("20181112", dt(2018, 11, 12)),
+            ("20182122", None),
+            ("181111", None),
+            ("181122", None),
+            ("182112", None),
+            ("181112", None),
+            ("182122", None),
+            ("111118", None),
+            ("112218", dt(2018, 11, 22)),
+            ("211218", None),
+            ("111218", None),
+            ("212218", None),
+            # dateutil.relativedelta
+            ("month+1", dt(2021, 1, 13, 9, 42, 53)),
+            ("month 1", dt(2021, 1, 13, 9, 42, 53)),
+            ("month-1", dt(2020, 11, 13, 9, 42, 53)),
+            ("month=1", dt(2020, 1, 13, 9, 42, 53)),
+            ("thurs+1", dt(2020, 12, 17, 9, 42, 53)),
+            ("thurs-1", dt(2020, 12, 10, 9, 42, 53)),
+            ("thurs=1", None),
+            ("month+1 month=1", dt(2021, 1, 13, 9, 42, 53)),
+            ("month=1 month+1", dt(2020, 2, 13, 9, 42, 53)),
+            ("bobit+1", None),
+        ],
+    )
+    def test_parse(self, value, expected):
+        parse_helper(DateTimeType, None, value, expected)
 
     def test_default_lookup(self):
         assert DateTimeType.default_lookup == "equals"
@@ -410,16 +437,16 @@ class TestDateTimeType:
 
 
 class TestDurationType:
-    def test_validate(self):
-        assert DurationType.parse("gt", "5 days", None) == (timedelta(days=5), None)
-        assert DurationType.parse("gt", "5 5:5", None) == (
-            timedelta(days=5, hours=5, minutes=5),
-            None,
-        )
-        assert DurationType.parse("gt", "5 dayss", None) == (None, ANY(str))
-        assert DurationType.parse("pontains", "5 days", None) == (None, ANY(str))
-        assert DurationType.parse("is_null", "IsNull", None) == (True, None)
-        assert DurationType.parse("is_null", "hello", None) == (None, ANY(str))
+    @pytest.mark.parametrize(
+        "value,expected",
+        [
+            ("5 days", timedelta(days=5)),
+            ("5 5:5", timedelta(days=5, hours=5, minutes=5)),
+            ("5 dayss", None),
+        ],
+    )
+    def test_parse(self, value, expected):
+        parse_helper(DurationType, None, value, expected)
 
     def test_default_lookup(self):
         assert DurationType.default_lookup == "equals"
@@ -434,21 +461,22 @@ class TestDurationType:
 
 
 class TestDateType:
-    def test_validate(self):
-        assert DateType.parse("gt", "2018-03-20T22:31:23", None) == (ANY(date), None)
-        assert DateType.parse("gt", "hello", None) == (None, ANY(str))
-        assert DateType.parse("pontains", "2018-03-20T22:31:23", None) == (
-            None,
-            ANY(str),
-        )
-        assert DateType.parse("is_null", "IsNull", None) == (True, None)
-        assert DateType.parse("is_null", "hello", None) == (None, ANY(str))
-        assert DateType.parse("gt", "today", None) == (ANY(date), None)
-
-        assert DateType.parse("gt", "11-22-2018", None) == (ANY(date), None)
-        assert DateType.parse("gt", "21-12-2018", None) == (ANY(date), None)
-        assert DateType.parse("gt", "11-12-2018", None) == (None, ANY(str))
-        assert DateType.parse("gt", "21-22-2018", None) == (None, ANY(str))
+    @time_machine.travel("2020-12-13 09:42:53", tick=False)
+    @pytest.mark.parametrize(
+        "value,expected",
+        [
+            ("2018-03-20T22:31:23", dt(2018, 3, 20).date()),
+            ("hello", None),
+            ("today", dt(2020, 12, 13).date()),
+            ("11-22-2018", dt(2018, 11, 22).date()),
+            ("21-12-2018", dt(2018, 12, 21).date()),
+            ("11-12-2018", None),
+            ("21-22-2018", None),
+            ("days+1", dt(2020, 12, 14).date()),
+        ],
+    )
+    def test_parse(self, value, expected):
+        parse_helper(DateType, None, value, expected)
 
     def test_default_lookup(self):
         assert DateType.default_lookup == "equals"
@@ -459,11 +487,16 @@ class TestDateType:
 
 
 class TestBooleanType:
-    def test_validate(self):
-        assert BooleanType.parse("equals", "True", None) == (True, None)
-        assert BooleanType.parse("equals", "False", None) == (False, None)
-        assert BooleanType.parse("equals", "hello", None) == (None, ANY(str))
-        assert BooleanType.parse("pontains", "True", None) == (None, ANY(str))
+    @pytest.mark.parametrize(
+        "value,expected",
+        [
+            ("True", True),
+            ("False", False),
+            ("hello", None),
+        ],
+    )
+    def test_parse(self, value, expected):
+        parse_helper(BooleanType, None, value, expected)
 
     def test_format(self):
         assert BooleanType.get_formatter(None)(None) is None
@@ -482,9 +515,15 @@ class TestStringChoiceType:
     def test_bad_value(self):
         assert StringChoiceType.get_formatter(self.choices)("x") == "x"
 
-    def test_validate(self):
-        assert NumberChoiceType.parse("equals", "B", self.choices) == ("b", None)
-        assert NumberChoiceType.parse("equals", "X", self.choices) == (None, ANY(str))
+    @pytest.mark.parametrize(
+        "value,expected",
+        [
+            ("B", "b"),
+            ("X", None),
+        ],
+    )
+    def test_parse(self, value, expected):
+        parse_helper(NumberChoiceType, self.choices, value, expected)
 
 
 class TestNumberChoiceType:
@@ -497,9 +536,15 @@ class TestNumberChoiceType:
     def test_bad_value(self):
         assert NumberChoiceType.get_formatter(self.choices)(6) == 6
 
-    def test_validate(self):
-        assert NumberChoiceType.parse("equals", "B", self.choices) == (2, None)
-        assert NumberChoiceType.parse("equals", "X", self.choices) == (None, ANY(str))
+    @pytest.mark.parametrize(
+        "value,expected",
+        [
+            ("B", 2),
+            ("X", None),
+        ],
+    )
+    def test_parse(self, value, expected):
+        parse_helper(NumberChoiceType, self.choices, value, expected)
 
 
 class TestHTMLType:
@@ -513,12 +558,15 @@ class TestIsNullType:
         assert IsNullType.get_formatter(IsNullType.choices)(False) == "NotNull"
         assert IsNullType.get_formatter(IsNullType.choices)(None) == "IsNull"
 
-    def test_validate(self):
-        assert IsNullType.parse("equals", "IsNull", IsNullType.choices) == (True, None)
-        assert IsNullType.parse("equals", "NotNull", IsNullType.choices) == (
-            False,
-            None,
-        )
+    @pytest.mark.parametrize(
+        "value,expected",
+        [
+            ("IsNull", True),
+            ("NotNull", False),
+        ],
+    )
+    def test_parse(self, value, expected):
+        parse_helper(IsNullType, IsNullType.choices, value, expected)
 
 
 class TestUnknownType:
@@ -527,6 +575,12 @@ class TestUnknownType:
 
 
 class TestArrayStringType:
-    def test_validate(self):
-        assert StringArrayType.parse("equals", '["X"]', None) == (["X"], None)
-        assert StringArrayType.parse("equals", '"X"', None) == (None, ANY(str))
+    @pytest.mark.parametrize(
+        "value,expected",
+        [
+            ('["X"]', ["X"]),
+            ('"X"', None),
+        ],
+    )
+    def test_parse(self, value, expected):
+        parse_helper(StringArrayType, None, value, expected)
